package ru.amse.pomelov.starbugs.client.gui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.util.EnumSet;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.border.TitledBorder;

import ru.amse.pomelov.starbugs.client.ClientDispatcher;
import ru.amse.pomelov.starbugs.client.gui.tools.AttackTool;
import ru.amse.pomelov.starbugs.client.gui.tools.BirthTool;
import ru.amse.pomelov.starbugs.client.gui.tools.BuildTool;
import ru.amse.pomelov.starbugs.units.Buildings;
import ru.amse.pomelov.starbugs.units.Edifices;
import ru.amse.pomelov.starbugs.units.Units;
import exceptions.ClientException;

public class ControlPanel extends JPanel {
	private static final long serialVersionUID = 1L;

	private class BirthAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		private Units myUnitType;
		
		public BirthAction(Units unitType) {
			myUnitType = unitType;
		}
		
		public void actionPerformed(ActionEvent e) {
			myClientDispatcher.setTool(new BirthTool(myUnitType));
		}
	}
	
	private class BuildAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		private Edifices myBuildingType;
		
		public BuildAction(Edifices buildingType) {
			myBuildingType = buildingType;
		}
		
		public void actionPerformed(ActionEvent e) {
			myClientDispatcher.setTool(new BuildTool(myBuildingType));
		}
	}

	private class AttackAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		
		public AttackAction() {
			putValue(AbstractAction.SMALL_ICON, IconFactory.createAttackIcon());
			putValue(AbstractAction.NAME, "Attack");
		}

		public void actionPerformed(ActionEvent e) {
			myClientDispatcher.setTool(new AttackTool());
		}
	}
	
	
	private class SurrenderAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public SurrenderAction() {
			putValue(AbstractAction.SMALL_ICON, IconFactory.createSurrenderIcon());
			putValue(AbstractAction.NAME, "Surrender");
		}
		
		public void actionPerformed(ActionEvent e) {
			if (JOptionPane.showConfirmDialog(ControlPanel.this,
					"Do you really want to surrender?", "Confirm surrender",
					JOptionPane.YES_NO_OPTION) == JOptionPane.OK_OPTION) {
				disableAllActions();
				cancel.actionPerformed(null);				
				try {
					myClientDispatcher.surrender();
				} catch (ClientException e1) {
					e1.showMessage(null);
				}
			}
		}
	}
	
	private class EndOfTurnAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		
		public EndOfTurnAction() {
			putValue(AbstractAction.SMALL_ICON, IconFactory.createEOTIcon());
			putValue(AbstractAction.NAME, "End of turn");
		}

		public void actionPerformed(ActionEvent e) {
			disableAllActions();
			cancel.actionPerformed(null);
			try {
				myClientDispatcher.endOfTurn();
			} catch (ClientException e1) {
				e1.showMessage(null);
			}
		}
	}
	
	private class CancelUnitSelectionAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		
		public CancelUnitSelectionAction() {
			putValue(AbstractAction.NAME, "Cancel selection");
		}
		
		public void actionPerformed(ActionEvent e) {
			turnButtonsGroup.clearSelection();
			myClientDispatcher.setTool(null);
		}
	}

	private final AbstractAction birthWorker = new BirthAction(Units.WORKER);
	private final AbstractAction birthSoldier = new BirthAction(Units.SOLDIER);
	private final AbstractAction birthArcher = new BirthAction(Units.ARCHER);
	private final AbstractAction birthRunner = new BirthAction(Units.RUNNER);
	private final AbstractAction birthBallista = new BirthAction(Units.BALLISTA);
	private final AbstractAction birthCatapult = new BirthAction(Units.CATAPULT);
	private final AbstractAction birthTank = new BirthAction(Units.TANK);
	
	private final AbstractAction buildBase = new BuildAction(Edifices.BASE_EDIFICE);
	private final AbstractAction buildLaboratory = new BuildAction(Edifices.LABORATORY);
	private final AbstractAction buildBarracks = new BuildAction(Edifices.BARRACK_EDIFICE);
	private final AbstractAction buildArchery = new BuildAction(Edifices.ARCHERY_EDIFICE);
	private final AbstractAction buildSmallWorkshop = new BuildAction(Edifices.SMALL_WORKSHOP);
	private final AbstractAction buildBigWorkshop = new BuildAction(Edifices.BIG_WORKSHOP);
	private final AbstractAction buildSmallFactory = new BuildAction(Edifices.SMALL_FACTORY);
	private final AbstractAction buildBigFactory = new BuildAction(Edifices.BIG_FACTORY);

	private final AbstractAction attack = new AttackAction();
	private final AbstractAction cancel = new CancelUnitSelectionAction();
	private final AbstractAction surrender = new SurrenderAction();
	private final AbstractAction endOfTurn = new EndOfTurnAction();
	
	public void disableAllActions() {
		birthWorker.setEnabled(false);
		birthSoldier.setEnabled(false);
		birthArcher.setEnabled(false);
		birthRunner.setEnabled(false);
		birthBallista.setEnabled(false);
		birthCatapult.setEnabled(false);
		birthTank.setEnabled(false);
		
		buildBase.setEnabled(false);
		buildLaboratory.setEnabled(false);
		buildBarracks.setEnabled(false);
		buildArchery.setEnabled(false);
		buildSmallWorkshop.setEnabled(false);
		buildBigWorkshop.setEnabled(false);
		buildSmallFactory.setEnabled(false);
		buildBigFactory.setEnabled(false);
		
		attack.setEnabled(false);
		surrender.setEnabled(false);
		endOfTurn.setEnabled(false);
		
	}

	public void enableAllowed(EnumSet<Units> allowedUnits,
			EnumSet<Buildings> allowedBuildings) {
		birthWorker.setEnabled(allowedUnits.contains(Units.WORKER));
		birthSoldier.setEnabled(allowedUnits.contains(Units.SOLDIER));
		birthArcher.setEnabled(allowedUnits.contains(Units.ARCHER));
		birthRunner.setEnabled(allowedUnits.contains(Units.RUNNER));
		birthBallista.setEnabled(allowedUnits.contains(Units.BALLISTA));
		birthCatapult.setEnabled(allowedUnits.contains(Units.CATAPULT));
		birthTank.setEnabled(allowedUnits.contains(Units.TANK));
		
		buildBase.setEnabled(allowedBuildings.contains(Buildings.BASE));
		buildLaboratory.setEnabled(allowedBuildings.contains(Buildings.LABORATORY));
		buildBarracks.setEnabled(allowedBuildings.contains(Buildings.BARRACKS));
		buildArchery.setEnabled(allowedBuildings.contains(Buildings.ARCHERY));
		buildSmallWorkshop.setEnabled(allowedBuildings.contains(Buildings.SMALL_WORKSHOP));
		buildBigWorkshop.setEnabled(allowedBuildings.contains(Buildings.BIG_WORKSHOP));
		buildSmallFactory.setEnabled(allowedBuildings.contains(Buildings.SMALL_FACTORY));
		buildBigFactory.setEnabled(allowedBuildings.contains(Buildings.BIG_FACTORY));
		
		attack.setEnabled(true);
		surrender.setEnabled(true);
		endOfTurn.setEnabled(true);
	}
	
	
	private final ButtonGroup turnButtonsGroup = new ButtonGroup();
	
	private void setButtonIcons(JToggleButton button, Units unitType) {
		button.setIcon(IconFactory.createUnitButtonIcon(unitType));
		button.setDisabledIcon(IconFactory.createUnitDisableButtonIcon(unitType));
		button.setPressedIcon(IconFactory.createUnitPressedButtonIcon(unitType));
		button.setSelectedIcon(IconFactory.createUnitPressedButtonIcon(unitType));
		turnButtonsGroup.add(button);
	}
	
	private JPanel createBirthPanel() {
		JPanel result = new JPanel(new GridLayout(3, 3, 3, 3));
		
		JToggleButton worker = new JToggleButton(birthWorker);
		setButtonIcons(worker, Units.WORKER);
		result.add(worker);
		
		JToggleButton soldier = new JToggleButton(birthSoldier);
		setButtonIcons(soldier, Units.SOLDIER);
		result.add(soldier);
		
		JToggleButton archer = new JToggleButton(birthArcher);
		setButtonIcons(archer, Units.ARCHER);
		result.add(archer);

		JToggleButton runner = new JToggleButton(birthRunner);
		setButtonIcons(runner, Units.RUNNER);
		result.add(runner);
		
		JToggleButton ballista = new JToggleButton(birthBallista);
		setButtonIcons(ballista, Units.BALLISTA);
		result.add(ballista);
		
		JToggleButton catapult = new JToggleButton(birthCatapult);
		setButtonIcons(catapult, Units.CATAPULT);
		result.add(catapult);
		
		JToggleButton tank = new JToggleButton(birthTank);
		setButtonIcons(tank, Units.TANK);
		result.add(tank);
		
		TitledBorder border = BorderFactory.createTitledBorder("Birth");
		border.setTitleJustification(TitledBorder.CENTER);
		result.setBorder(border);		
		return result;
	}

	private void setButtonIcons(JToggleButton button, Buildings buildingType) {
		button.setIcon(IconFactory.createBuildingButtonIcon(buildingType));
		button.setDisabledIcon(IconFactory.createBuildingDisableButtonIcon(buildingType));
		button.setPressedIcon(IconFactory.createBuildingPressedButtonIcon(buildingType));
		button.setSelectedIcon(IconFactory.createBuildingPressedButtonIcon(buildingType));
		turnButtonsGroup.add(button);
	}
	private JPanel createBuildPanel() {
		JPanel result = new JPanel(new GridLayout(3, 3, 3, 3));
		
		JToggleButton base = new JToggleButton(buildBase);
		setButtonIcons(base, Buildings.BASE);
		result.add(base);
		
		JToggleButton laboratory = new JToggleButton(buildLaboratory);
		setButtonIcons(laboratory, Buildings.LABORATORY);
		result.add(laboratory);
		
		JToggleButton barracks = new JToggleButton(buildBarracks);
		setButtonIcons(barracks, Buildings.BARRACKS);
		result.add(barracks);
		
		JToggleButton archery = new JToggleButton(buildArchery);
		setButtonIcons(archery, Buildings.ARCHERY);
		result.add(archery);

		JToggleButton smallWorkshop = new JToggleButton(buildSmallWorkshop);
		setButtonIcons(smallWorkshop, Buildings.SMALL_WORKSHOP);
		result.add(smallWorkshop);
		
		JToggleButton bigWorkshop = new JToggleButton(buildBigWorkshop);
		setButtonIcons(bigWorkshop, Buildings.BIG_WORKSHOP);
		result.add(bigWorkshop);
		
		JToggleButton smallFactory = new JToggleButton(buildSmallFactory);
		setButtonIcons(smallFactory, Buildings.SMALL_FACTORY);
		result.add(smallFactory);
		
		JToggleButton tank = new JToggleButton(buildBigFactory);
		setButtonIcons(tank, Buildings.BIG_FACTORY);
		result.add(tank);
		
		TitledBorder border = BorderFactory.createTitledBorder("Build");
		border.setTitleJustification(TitledBorder.CENTER);
		result.setBorder(border);		
		return result;
	}
	
	private JPanel createMovesPanel() {
		
		JPanel result = new JPanel(new GridLayout(2, 1));
		
		result.add(createBirthPanel());
		result.add(createBuildPanel());
		return result;
	}
	
	private JPanel createMovesButtons() {
		JPanel result = new JPanel(new GridLayout(4, 1));
		
		
		JToggleButton attackButton = new JToggleButton(attack);
		turnButtonsGroup.add(attackButton);
		result.add(attackButton);
		result.add(new JButton(surrender));
		result.add(new JButton(endOfTurn));
		
		return result;
	}
	
	private JPanel createControls() {
		JPanel result = new JPanel();
		result.setLayout(new BoxLayout(result, BoxLayout.PAGE_AXIS));
		
		result.add(createMovesPanel());
		result.add(createMovesButtons());
		
		return result;
	}
	
	private final ClientDispatcher myClientDispatcher;
	
	public ControlPanel(ClientDispatcher clientDispatcher) {
		myClientDispatcher = clientDispatcher;
		setLayout(new BorderLayout());
		add(createControls(), BorderLayout.NORTH);
		disableAllActions();
		setBorder(BorderFactory.createTitledBorder("Control panel"));
	}
}
