package ru.amse.pomelov.starbugs.client.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.List;

import javax.swing.JViewport;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ru.amse.pomelov.starbugs.client.ClientDispatcher;
import ru.amse.pomelov.starbugs.config.Options;
import ru.amse.pomelov.starbugs.units.AbstractUnit;
import ru.amse.pomelov.starbugs.util.FieldPosition;

public class MiniMapView extends GameFieldView {
	private static final long serialVersionUID = 1L;

	private int myWidth;
	private int myHeight;
	private final JViewport myViewport;
	
	private class MiniMapNavigationListener extends MouseAdapter {
		
		private void moveView(MouseEvent e) {
			Dimension viewSize = myViewport.getViewSize();
			Dimension extSize = myViewport.getExtentSize();
			int w = Math.min(myWidth - 3, extSize.width * (myWidth - 2) / viewSize.width);
			int h = Math.min(myHeight - 3, extSize.height * (myHeight - 2) / viewSize.height);

			int x = (e.getX() - w / 2 - 1) * viewSize.width / (myWidth - 2);
			int y = (e.getY() - h / 2 - 1) * viewSize.height / (myHeight - 2);
			
			x = (x < 0) ? 0 : x;
			y = (y < 0) ? 0 : y;
			
			x = (x + extSize.width > viewSize.width) ? viewSize.width - extSize.width : x;
			y = (y + extSize.height > viewSize.height) ? viewSize.height - extSize.height : y;
			
			myViewport.setViewPosition(new Point(x, y));			
		}
		
		@Override
		public void mousePressed(MouseEvent e) {
			moveView(e);
		}

		@Override
		public void mouseDragged(MouseEvent e) {
			moveView(e);
		}

	}
	
	public MiniMapView(ClientDispatcher clientDispatcher, JViewport viewport) {
		super(clientDispatcher);
		myViewport = viewport;
		int w = (200 / myGameField.getWidth()) * myGameField.getWidth();
		int h = (200 / myGameField.getHeight()) * myGameField.getHeight();
		myWidth = 2 + ((w == 0) ? 200 : w);
		myHeight = 2 + ((h == 0) ? 200 : h);
		myBuffer = new BufferedImage(myWidth, myHeight, BufferedImage.TYPE_4BYTE_ABGR);
		paintToBuffer();
		myViewport.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				MiniMapView.this.repaint();
			}
		});
		addMouseListener(new MiniMapNavigationListener());
		addMouseMotionListener(new MiniMapNavigationListener());
		setPreferredSize(new Dimension(202, 202));
		setMinimumSize(new Dimension(myWidth, myHeight));
		setMaximumSize(new Dimension(myWidth, myHeight));
	}

	
	private BufferedImage myBuffer;
	public void paintToBuffer() {
		Graphics g = myBuffer.getGraphics();
		
		g.setColor(Options.BACKGROUND_COLOR);
		g.fillRect(0, 0, myWidth - 1, myHeight - 1);
		
		g.setColor(Color.BLACK);
		g.drawRect(0, 0, myWidth - 1, myHeight - 1);
		
		for (int i = 1; i < myWidth - 1; i++) {
			for (int j = 1; j < myHeight - 1; j++) {
				int x = (i - 1) * fieldWidth / (myWidth - 2);
				int y = (j - 1) * fieldHeight / (myHeight - 2);
				AbstractUnit u = myGameField.getUnit(FieldPosition.createPoint(x, y));
				if (u == null) {
					g.setColor(Color.WHITE);
				} else {
					g.setColor(Options.getColor(u.getPlayer()));
				}
				g.drawLine(i, j, i, j);
			}
		}
	}
	
	@Override
	public void paint(Graphics g) {
		
		g.drawImage(myBuffer, 0, 0, this);
		
		Point viewPos = myViewport.getViewPosition();
		Dimension viewSize = myViewport.getViewSize();
		int x = viewPos.x * (myWidth - 2) / viewSize.width + 1;
		int y = viewPos.y * (myHeight - 2) / viewSize.height + 1;
		
		Dimension extSize = myViewport.getExtentSize();
		int w = Math.min(myWidth - 3, extSize.width * (myWidth - 2) / viewSize.width);
		int h = Math.min(myHeight - 3, extSize.height * (myHeight - 2) / viewSize.height);
		
		g.setColor(Color.BLACK);
		g.drawRect(x, y, w, h);
		
	}
	
	@Override
	public void putUnits(List<AbstractUnit> unitList) {
		super.putUnits(unitList);
		paintToBuffer();
	}

	@Override
	public void removeUnit(FieldPosition coords) {
		super.removeUnit(coords);
		paintToBuffer();
	}
	
	
}
