package ru.amse.pomelov.starbugs.client.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.ImageObserver;

import javax.swing.ImageIcon;
import javax.swing.JViewport;

import ru.amse.pomelov.starbugs.client.ClientDispatcher;
import ru.amse.pomelov.starbugs.config.Options;
import ru.amse.pomelov.starbugs.units.AbstractUnit;
import ru.amse.pomelov.starbugs.util.FieldPosition;

public class PlayFieldView extends GameFieldView {
	private static final long serialVersionUID = 1L;

	public static final int DEFAULT_CELL_SIZE = 33;
	
	private class GameFieldMouseListener extends MouseAdapter {
		@Override
		public void mousePressed(MouseEvent e) {
			if (e.getButton() == MouseEvent.BUTTON1) {
				int mx = e.getX();
				int my = e.getY();
				
				FieldPosition object = FieldPosition.createPoint(mx / myCellSize, my / myCellSize);
				myClientDispatcher.clicked(object);
				mySelectedCell = myClientDispatcher.getSelected();

				PlayFieldView.this.repaint();

			}
		}

		@Override
		public void mouseMoved(MouseEvent e) {
			cursorX = e.getX() / myCellSize;
			cursorY = e.getY() / myCellSize;
			PlayFieldView.this.repaint();
		}

		@Override
		public void mouseExited(MouseEvent e) {
			cursorX = -1;
			cursorY = -1;
		}
		
		
	}
	
	private int myCellSize = DEFAULT_CELL_SIZE;

	private FieldPosition mySelectedCell = null;
	
	private final JViewport myViewport;
	
	private int cursorX = -1;
	private int cursorY = -1;
	
	
	public PlayFieldView(ClientDispatcher clientDispatcher, JViewport viewport) {
		super(clientDispatcher);
		
		myViewport = viewport;
		
		addMouseListener(new GameFieldMouseListener());
		addMouseMotionListener(new GameFieldMouseListener());
		
		myClientDispatcher.setSelectionListener(new SelectionListener(){
			public void selectionGained(FieldPosition object) {
				mySelectedCell = object;
				repaint();
			}
			public void selectionLost() {
				mySelectedCell = null;
				repaint();
			}
		});

		setPreferredSize(new Dimension(fieldWidth * myCellSize + 1, fieldHeight * myCellSize + 1));
		
	}

	@Override
	public void paintComponent(Graphics g) {
		
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, getWidth(), getHeight());
		g.setColor(Color.LIGHT_GRAY);
		

		
		for (int i = 0; i <= fieldWidth; i++) {
			g.drawLine(i * myCellSize, 0, i * myCellSize, fieldHeight * myCellSize);
		}

		
		for (int i = 0; i <= fieldHeight; i++) {
			g.drawLine(0, i * myCellSize, fieldWidth * myCellSize, i * myCellSize);
		}

		Point viewPosition = myViewport.getViewPosition();
		Dimension extentSize = myViewport.getExtentSize();
		
		int x1 = viewPosition.x;
		int y1 = viewPosition.y;
		int x2 = x1 + extentSize.width;
		int y2 = y1 + extentSize.height;
		
		x1 /= myCellSize;
		y1 /= myCellSize;
		x2 /= myCellSize;
		y2 /= myCellSize;

		x1--;
		y1--;
		x2++;
		y2++;
		
		x1 = (x1 < 0) ? 0 : x1;
		y1 = (y1 < 0) ? 0 : y1;
		
		x2 = (x2 > fieldWidth) ? fieldWidth: x2;
		y2 = (y2 > fieldHeight) ? fieldHeight: y2;
		
		
		for (int i = x1; i < x2; i++) {
			for (int j = y1; j < y2; j++) {
				AbstractUnit u = myGameField.getUnit(FieldPosition.createPoint(i, j));
				
				if (u != null) {
					ImageIcon unitIcon = u.acceptIconFacory();
					ImageObserver obs = unitIcon.getImageObserver();
					Image image = unitIcon.getImage();
					int dx1 = i * myCellSize + 1;
					int dy1 = j * myCellSize + 1;
					int dx2 = (i + 1) * myCellSize;
					int dy2 = (j + 1) * myCellSize;
					int imW = image.getWidth(obs);
					int imH = image.getHeight(obs);

					g.setColor(Options.getColor(u.getPlayer()));
					g.fillRect(dx1, dy1, myCellSize - 1, myCellSize - 1);
					g.drawImage(image, dx1, dy1, dx2, dy2, 0, 0, imW, imH, obs);
				}
				
			}
		}
		
		

		
		if (mySelectedCell != null) {
			int x = mySelectedCell.getX();
			int y = mySelectedCell.getY();
			drawFrame(g, x, y, Color.CYAN);
		}
		
		if ((myClientDispatcher != null) && (cursorX != -1) && (cursorY != -1)) {
			myClientDispatcher.drawToolTip(g, cursorX, cursorY, myCellSize);
		}
	}

	private void drawFrame(Graphics g, int x, int y, Color color) {
		if ((x < 0) || (y < 0) || (x >= myGameField.getWidth()) || (y >= myGameField.getHeight())) {
			return;
		}
		g.setColor(color);
		
		g.drawRect(x * myCellSize, y * myCellSize, myCellSize, myCellSize);
		g.drawRect(x * myCellSize - 1, y * myCellSize - 1, myCellSize + 2, myCellSize + 2);
		g.drawRect(x * myCellSize + 1, y * myCellSize + 1, myCellSize - 2, myCellSize - 2);
	}
	
	public int getCellSize() {
		return myCellSize;
	}
	
	public void setCellSize(int cellSize) {
		myCellSize = cellSize;
	}
	
	public void updatePreferredSize() {
		setPreferredSize(new Dimension(fieldWidth * myCellSize + 1, fieldHeight * myCellSize + 1));
		setSize(fieldWidth * myCellSize + 1, fieldHeight * myCellSize + 1);
	}
	
}
