package ru.amse.pomelov.starbugs.client.gui.frames;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.AbstractListModel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import ru.amse.pomelov.starbugs.remote.GameServer;
import ru.amse.pomelov.starbugs.remote.MainServer;
import ru.amse.pomelov.starbugs.util.GameProperties;

public class GameSelectionDialog extends JDialog {
	
	
	private class GameListModel extends AbstractListModel {
		private static final long serialVersionUID = 1L;
		
		
		public Object getElementAt(int index) {
			return gameServers.get(index);
		}

		public int getSize() {
			return gameServers.size();
		}
		
		public void refresh() {
		    try {
				MainServer server = (MainServer)Naming.lookup(serverURL.getText());
				gameServers.clear();
				for (String gameName : server.viewGames()) {
					GameServer game = ((GameServer) Naming.lookup(gameName));
					gameServers.add(game);
				}
				fireContentsChanged(this, 0, getSize());
			} catch (MalformedURLException e1) {
				JOptionPane.showMessageDialog(GameSelectionDialog.this, "Main server URL is mal formed.");
			} catch (RemoteException e1) {
				JOptionPane.showMessageDialog(GameSelectionDialog.this, "Remote exception.");
			} catch (NotBoundException e1) {
				JOptionPane.showMessageDialog(GameSelectionDialog.this, "Main server is not bound.");
			}
		}
	}

	private class GameListSelectionListener implements ListSelectionListener {
		public void valueChanged(ListSelectionEvent e) {
			int index = gameList.getSelectedIndex();
			selectedGame = (GameServer) gameListModel.getElementAt(index);
			refreshDetails();
			joinButton.setEnabled(true);
		}
	}
	
	private class GameListDoubleClickListener extends MouseAdapter {
		private int x = -1;
		private int y = -1;
		private long d = 0;
		
		@Override
		public void mouseClicked(MouseEvent e) {
			if (e.getButton() == MouseEvent.BUTTON1) {
				if (d == 0) {
					x = e.getX();
					y = e.getY();
					d = new Date().getTime();
				} else {
					long d1 = new Date().getTime();
					if ((x == e.getX()) && (y == e.getY()) && ((d1 - d) < 200)) {
						new JoinAction().actionPerformed(null);
					}
					x = e.getX();
					y = e.getY();
					d = d1;
				}
			}
		}
		
	}
	
	private class RefreshAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public RefreshAction() {
			putValue(AbstractAction.NAME, "Refresh");
		}
		
		public void actionPerformed(ActionEvent e) {
			gameListModel.refresh();
		}
	}

	private class JoinAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public JoinAction() {
			putValue(AbstractAction.NAME, "Select");
		}
		
		public void actionPerformed(ActionEvent e) {
			GameSelectionDialog.this.dispose();
		}
	}
	
	private class CancelAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		
		public CancelAction() {
			putValue(AbstractAction.NAME, "Cancel");
		}
		
		public void actionPerformed(ActionEvent e) {
			selectedGame = null;
			GameSelectionDialog.this.dispose();
		}
	}
	
	
	
	private static final long serialVersionUID = 1L;

	private JTextField serverURL = new JTextField("rmi://localhost/server");
	
	private List<GameServer> gameServers = new ArrayList<GameServer>();
	private GameListModel gameListModel = new GameListModel();
	private JList gameList = new JList(gameListModel);
	private void initGameList() {
		gameList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		gameList.addListSelectionListener(new GameListSelectionListener());
		gameList.setCellRenderer(new GameListCellRenderer());
	}
	
	
	private JTextField detailsGameName = new JTextField();
	private JTextField detailsFieldSize = new JTextField();
	private JTextField detailsPlayersNum = new JTextField();
	private void initDetailsFields() {
		detailsGameName.setEditable(false);
		detailsFieldSize.setEditable(false);
		detailsPlayersNum.setEditable(false);	
	}

	private JButton joinButton = new JButton(new JoinAction());
	private JButton cancelButton = new JButton(new CancelAction());
	private void initButtons() {
		joinButton.setEnabled(false);
	}
	
	private void refreshDetails() {
		if (selectedGame != null) {
			try {
			detailsGameName.setText(selectedGame.getProperties().name);
			GameProperties props = selectedGame.getProperties();
			int cur = selectedGame.getRegistredPlayerNum();
			detailsFieldSize.setText("[" + props.width + " x " + props.height + "]");
			detailsPlayersNum.setText(" " + cur + " / " + props.playerNum);
			} catch (RemoteException e) {
				detailsGameName.setText("");
				detailsFieldSize.setText("");
				detailsPlayersNum.setText("");
			}			
		} else {
			detailsGameName.setText("");
			detailsFieldSize.setText("");
			detailsPlayersNum.setText("");
		}	
	
	}
	
	private JPanel createDetailsPanel() {
		JPanel details = new JPanel(new GridLayout(3, 2));
		refreshDetails();
		details.add(new JLabel("Game name: "));
		details.add(detailsGameName);
		details.add(new JLabel("Field size: "));
		details.add(detailsFieldSize);
		details.add(new JLabel("Players: "));
		details.add(detailsPlayersNum);

		
		
		JPanel result = new JPanel(new BorderLayout());
		result.add(details, BorderLayout.NORTH);
		
		TitledBorder border = BorderFactory.createTitledBorder("Details");
		border.setTitleJustification(TitledBorder.CENTER);
		result.setBorder(border);
		
		return result;
	}
	


	private JPanel createServerURLPanel() {
		JPanel result = new JPanel(new BorderLayout());
		result.add(new JLabel(" Server URL : "), BorderLayout.WEST);
		result.add(serverURL, BorderLayout.CENTER);
		result.add(new JButton(new RefreshAction()), BorderLayout.EAST);
		return result;
	}

	private JPanel createButtonsPanel() {
		JPanel result = new JPanel(new GridLayout(1, 2, 5, 5));
		result.setBorder(new EmptyBorder(5, 5, 5, 5));
		result.add(joinButton);
		result.add(cancelButton);		
		return result;
	}	
	
	private JPanel createGamesPanel() {
		JPanel result = new JPanel();
		
		result.setLayout(new BorderLayout());
		result.add(gameList, BorderLayout.CENTER);
		
		result.add(createButtonsPanel(), BorderLayout.SOUTH);

		TitledBorder border = BorderFactory.createTitledBorder("Games");
		border.setTitleJustification(TitledBorder.CENTER);
		gameList.setBorder(border);

		
		return result;
	}
	
	private GameSelectionDialog(JFrame owner) {
		super(owner, "Select game", true);
		setSize(640, 480);
		setLocation(360, 220);
		setResizable(false);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		
		JPanel panel = new JPanel(new BorderLayout());
		getContentPane().add(panel);
		
		gameList.addMouseListener(new GameListDoubleClickListener());
		
		initGameList();
		initDetailsFields();
		initButtons();
		
		
		
		panel.add(createServerURLPanel(), BorderLayout.NORTH);
		panel.add(createGamesPanel(), BorderLayout.EAST);
		panel.add(createDetailsPanel(), BorderLayout.CENTER);
		
		panel.getInputMap().put(KeyStroke.getKeyStroke((char)27), "cancel");
		panel.getActionMap().put("cancel", new CancelAction());
		
		new RefreshAction().actionPerformed(null);
		
		setVisible(true);
	}
	
	private static GameServer selectedGame = null;
	
	public static GameServer selectGame(JFrame owner) {
		new GameSelectionDialog(owner);
		GameServer result = selectedGame;
		selectedGame = null;
		
		return result;
	}
	
}

class GameListCellRenderer implements ListCellRenderer {
	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
		GameServer server = (GameServer) value;
		try {
			JPanel p = new JPanel();
			p.setLayout(new BoxLayout(p, BoxLayout.LINE_AXIS));
			if (cellHasFocus) {
				p.setBorder(BorderFactory.createLineBorder(Color.GRAY));
			}
			
			if (isSelected) {
				p.setBackground(Color.LIGHT_GRAY);
			} else {
				p.setBackground(Color.WHITE);
			}
			
			p.add(new JLabel(server.getProperties().name));
			return p;
		} catch (RemoteException e) {
		}
		return new JLabel("Error");
	}
}