package ru.amse.pomelov.starbugs.server;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;

import ru.amse.pomelov.starbugs.config.Options;
import ru.amse.pomelov.starbugs.remote.GameServer;
import ru.amse.pomelov.starbugs.remote.RemoteAdapter;
import ru.amse.pomelov.starbugs.remote.RemoteGameServerListener;
import ru.amse.pomelov.starbugs.turns.AbstractTurn;
import ru.amse.pomelov.starbugs.units.AbstractUnit;
import ru.amse.pomelov.starbugs.units.Buildings;
import ru.amse.pomelov.starbugs.units.Units;
import ru.amse.pomelov.starbugs.util.FieldPosition;
import ru.amse.pomelov.starbugs.util.GameProperties;
import ru.amse.pomelov.starbugs.util.UserID;

public class GameServerImpl implements GameServer {
	
	private class Sayer implements Runnable {
		private final RemoteGameServerListener l;
		private final List<AbstractUnit> list;
		private final List<FieldPosition> list2;
		
		private final int winner;

		public Sayer(RemoteGameServerListener l, int winner) {
			this.l = l;
			this.list = null;
			this.list2 = null;
			this.winner = winner;
		}
		
		public Sayer(RemoteGameServerListener l, List<AbstractUnit> list) {
			this.l = l;
			this.list = list;
			this.list2 = null;
			this.winner = 0;
		}

		public Sayer(List<FieldPosition> list, RemoteGameServerListener l) {
			this.l = l;
			this.list = null;
			this.list2 = list;
			this.winner = 0;
		}
		public void run() {
			try {
				if (list != null) {
					l.createUnits(list);
				} else if (list2 != null) {
					l.removeUnits(list2);
				} else {
					l.endOfGame(winner);
					mainServerListener.killGame(myGameName);
				}
			} catch (RemoteException e) {
			}
		}
	}
	
	
	private class RemoteListener extends RemoteAdapter implements Remote {
		@Override
		public void createUnits(List<AbstractUnit> list) throws RemoteException {
			for (RemoteGameServerListener l : myPlayerListeners) {
				new Thread(new Sayer(l, list)).start();
			}
			for (RemoteGameServerListener l : myMonitorListeners) {
				new Thread(new Sayer(l, list)).start();
			}
		}
		
		@Override
		public void removeUnits(List<FieldPosition> list) throws RemoteException {
			for (RemoteGameServerListener l : myPlayerListeners) {
				new Thread(new Sayer(list, l)).start();
			}
			for (RemoteGameServerListener l : myMonitorListeners) {
				new Thread(new Sayer(list, l)).start();
			}
		}
		@Override
		public void endOfGame(int winner) throws RemoteException {
			for (RemoteGameServerListener l : myPlayerListeners) {
				new Thread(new Sayer(l, winner)).start();
			}
			for (RemoteGameServerListener l : myMonitorListeners) {
				new Thread(new Sayer(l, winner)).start();
			}
		}
		
	}
	
	private static final Random rnd = new Random();
	
	private final Map<Integer, Integer> myPlayerIDs = new HashMap<Integer, Integer>(); 
	
	private GameDispatcher myGameDispatcher;
	private final int myID;
	private final String myGameName;
	private boolean registred = false;
	private boolean started = false;
	private GameProperties myGameProperties;
	private int alreadyRegistered;
	
	private final List<RemoteGameServerListener> myPlayerListeners = new LinkedList<RemoteGameServerListener>();
	private final List<RemoteGameServerListener> myMonitorListeners = new LinkedList<RemoteGameServerListener>();
	
	private final GameKiller mainServerListener;
	
	public GameServerImpl(String name, int ID, GameKiller l) {
		myID = ID << 16;
		myGameName = name;
		mainServerListener = l;
	}

	private void notifyAllGameStarted() {
		new Thread(new Runnable() {
			public void run() {
				for (RemoteGameServerListener l : GameServerImpl.this.myPlayerListeners) {
					try {
						l.gameStarted();
					} catch (RemoteException e) {
					}
				}
				for (RemoteGameServerListener l : GameServerImpl.this.myMonitorListeners) {
					try {
						l.gameStarted();
					} catch (RemoteException e) {
					}
				}
			}
		}, "Game start notificator").start();
	}
	
	private void notifyAllPlayerJoined() {
			for (RemoteGameServerListener l : GameServerImpl.this.myPlayerListeners) {
				try {
					l.playerJoined();
				} catch (RemoteException e) {
				}
			}
			for (RemoteGameServerListener l : GameServerImpl.this.myMonitorListeners) {
				try {
					l.playerJoined();
				} catch (RemoteException e) {
				}
			}
	}	
	
	public UserID join(RemoteGameServerListener listener) throws RemoteException {
		if (!registred) {
			return null;
		}
		
		if (started) {
			return null;
		}
		
		UserID newID;
		do {
			newID = new UserID(myID, rnd.nextInt(), alreadyRegistered + 1);
		} while (myPlayerIDs.containsValue(newID));
		
		myPlayerIDs.put(myPlayerIDs.size() + 1, newID.userID);
		
		
		if (listener != null) {
			myPlayerListeners.add(listener);
		}
		
		alreadyRegistered++;
		notifyAllPlayerJoined();
		if (alreadyRegistered == myGameProperties.playerNum) {
			started = true;
			
			notifyAllGameStarted();
			
			RemoteGameServerListener l = myPlayerListeners.get(0);
			if (l != null) {
				l.yourTurn(Options.maxTurnTime);
			}
		}
		
		return newID;
	}

	
	

	public void monitor(RemoteGameServerListener listener) throws RemoteException {
		myMonitorListeners.add(listener);
	}

	public boolean registerGame(GameProperties props) throws RemoteException {
		if (registred) {
			return false;
		}
		registred = true;
		
		alreadyRegistered = 0;
		myGameDispatcher = new GameDispatcher(props, this, new RemoteListener());
		myGameProperties = props;
		
		return true;
		
	}
	
	public int makeTurn(UserID playerID, AbstractTurn turn) throws RemoteException {
		if (!started) {
			return -1;
		}
		
		if (playerID == null) {
			return -1;
		}
		
		if (playerID.gameServerID != myID) {
			return -1;
		}
		
		if (playerID.userID != myPlayerIDs.get(playerID.playerNum)) {
			return -1;
		}
		
		turn.setPlayer(playerID.playerNum);
		
		return myGameDispatcher.proceedTurn(turn);
	}

	public void nextPlayer(int player, int timeAvailable) throws RemoteException {
		RemoteGameServerListener listener = myPlayerListeners.get(player);
		if (listener != null) {
			listener.yourTurn(timeAvailable);
		}
	}

	public GameProperties getProperties() throws RemoteException {
		return myGameProperties;
	}

	public int getRegistredPlayerNum() throws RemoteException {
		return alreadyRegistered;
	}
	
	public boolean hasVacancy() throws RemoteException {
		return myGameProperties.playerNum > alreadyRegistered;
	}
	
	public GameField getGameField() throws RemoteException {
		return myGameDispatcher.getGameField();
	}

	public EnumSet<Buildings> getAllowedBuildings() throws RemoteException {
		return myGameDispatcher.getAllowedBuildings();
	}

	public EnumSet<Units> getAllowedUnits() throws RemoteException {
		return myGameDispatcher.getAllowedUnits();
	}
	
	public void forceKill() throws RemoteException{
		for (RemoteGameServerListener l : GameServerImpl.this.myPlayerListeners) {
			try {
				l.endOfGame(-1);
			} catch (RemoteException e) {
			}
		}
		for (RemoteGameServerListener l : GameServerImpl.this.myMonitorListeners) {
			try {
				l.endOfGame(-1);
			} catch (RemoteException e) {
			}
		}
	}
}
