package ru.amse.pomelov.starbugs.server;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Random;
import java.util.Set;

import ru.amse.pomelov.starbugs.remote.GameServer;
import ru.amse.pomelov.starbugs.remote.MainServer;
import ru.amse.pomelov.starbugs.util.GameProperties;

public class MainServerImpl implements MainServer {

	public static final int MAX_GAMES = 100;
	
	private static final Random rnd = new Random();

	private Set<String> activeGames = new HashSet<String>();

	private final String bind;
	
	public MainServerImpl(String bind) {
		this.bind = bind;
	}
	
	private class MyGameKiller implements GameKiller {
		public void killGame(String gameName){
			MainServerImpl.this.activeGames.remove(gameName);
		}
	}
	
	
	public String registerGame(GameProperties props) throws RemoteException {
		
		String url = bind + "" + props.name.hashCode();
		if (activeGames.contains(url)) {
			return null;
		}
		activeGames.add(url);

		try {
			GameServer gameServer = new GameServerImpl(url, rnd.nextInt(1 << 16), new MyGameKiller());
			UnicastRemoteObject.exportObject(gameServer);
			Naming.rebind(url, gameServer);
			return gameServer.registerGame(props) ? url : null;

		} catch (RemoteException e) {
		} catch (MalformedURLException e) {
		}
		return null;
	}

	
	public Collection<String> viewGames() throws RemoteException {
		return new LinkedList<String>(activeGames);
	}
	
	public void forceKill() {
		for (String s : activeGames) {
			try {
				GameServer gameServer = (GameServer) Naming.lookup(s);
				gameServer.forceKill();
				Naming.unbind(s);
			} catch (MalformedURLException e) {
			} catch (RemoteException e) {
			} catch (NotBoundException e) {
			}
			
		}
	}
}