package ru.amse.pomelov.starbugs.server.testing;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.util.LinkedList;
import java.util.List;

import ru.amse.pomelov.starbugs.server.GameDispatcher;
import ru.amse.pomelov.starbugs.server.GameField;
import ru.amse.pomelov.starbugs.turns.AbstractTurn;
import ru.amse.pomelov.starbugs.turns.BirthTurn;
import ru.amse.pomelov.starbugs.units.Building;
import ru.amse.pomelov.starbugs.units.Buildings;
import ru.amse.pomelov.starbugs.units.Unit;
import ru.amse.pomelov.starbugs.units.Units;
import ru.amse.pomelov.starbugs.util.FieldPosition;
import ru.amse.pomelov.starbugs.util.GameProperties;

public class TestCreator {

	private static final File tests = new File("tests");

	private TestCreator() {
	}
	
	public static boolean addTest(String name, GameDispatcher start,
			List<AbstractTurn> turns, GameField finish) {

		File test = new File(tests, name);

		if (test.exists()) {
			return false;
		}

		if (!test.mkdir()) {
			return false;
		}

		File startFile = new File(test, "start.dat");
		File turnsFile = new File(test, "turns.dat");
		File finishFile = new File(test, "finish.dat");

		try {
			File configFile = new File(test, "config.cfg");
			configFile.createNewFile();

			OutputStream os = new FileOutputStream(startFile);
			start.SaveGame(os);
			os.close();

			os = new FileOutputStream(finishFile);
			finish.accept(new Saver(os));
			os.close();

			os = new FileOutputStream(turnsFile);
			
			ObjectOutputStream out = new ObjectOutputStream(os);

			int turnsNum = turns.size();
			out.writeInt(turnsNum);
			for (AbstractTurn t : turns) {
				out.writeObject(t);
			}
			os.close();

			return true;

		} catch (IOException e) {
			e.printStackTrace();
		}

		return false;
	}

	public static void main(String[] args) {
		FieldPosition p11 = FieldPosition.createPoint(1, 1); 
		FieldPosition p22 = FieldPosition.createPoint(2, 2); 
		FieldPosition p23 = FieldPosition.createPoint(2, 3); 
		FieldPosition p24 = FieldPosition.createPoint(2, 4);
		FieldPosition p25 = FieldPosition.createPoint(2, 5);
		FieldPosition p88 = FieldPosition.createPoint(8, 8);
		
		
		GameField start = new GameField(10, 10);
		GameDispatcher gd = new GameDispatcher(new GameProperties(10, 10, 2, ""), null, null);
		start.putUnit(new Building(p11, 1, Buildings.BASE));
		
		List<AbstractTurn> turns = new LinkedList<AbstractTurn>();
		turns.add(new BirthTurn(p22, Units.WORKER));
		turns.add(new BirthTurn(p23, Units.WORKER));
		turns.add(new BirthTurn(p24, Units.WORKER));
		turns.add(new BirthTurn(p25, Units.SOLDIER));
		

		
		GameField finish = new GameField(10, 10);
		finish.putUnit(new Building(p88, 2, Buildings.BASE));
		finish.putUnit(new Building(p11, 1, Buildings.BASE));
		finish.putUnit(new Unit(p22, 1, Units.WORKER));
		finish.putUnit(new Unit(p23, 1, Units.WORKER));
		finish.putUnit(new Unit(p24, 1, Units.WORKER));
		
		addTest("ImpossibleBirth", gd, turns, finish);
	}
}
